@@ -1,31 +1,30 @@
 /**************************************************************************
  *
- * Copyright (c) 2000-2003 Intel Corporation 
- * All rights reserved. 
- * Copyright (C) 2012 France Telecom All rights reserved.
+ * Copyright (c) 2000-2003 Intel Corporation
+ * All rights reserved.
  *
- * Redistribution and use in source and binary forms, with or without 
- * modification, are permitted provided that the following conditions are met: 
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions are met:
  *
- * - Redistributions of source code must retain the above copyright notice, 
- * this list of conditions and the following disclaimer. 
- * - Redistributions in binary form must reproduce the above copyright notice, 
- * this list of conditions and the following disclaimer in the documentation 
- * and/or other materials provided with the distribution. 
- * - Neither name of Intel Corporation nor the names of its contributors 
- * may be used to endorse or promote products derived from this software 
+ * - Redistributions of source code must retain the above copyright notice,
+ * this list of conditions and the following disclaimer.
+ * - Redistributions in binary form must reproduce the above copyright notice,
+ * this list of conditions and the following disclaimer in the documentation
+ * and/or other materials provided with the distribution.
+ * - Neither name of Intel Corporation nor the names of its contributors
+ * may be used to endorse or promote products derived from this software
  * without specific prior written permission.
- * 
- * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
- * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
- * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR 
- * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL INTEL OR 
- * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
- * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
- * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR 
- * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY 
+ *
+ * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
+ * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
+ * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
+ * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL INTEL OR
+ * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
+ * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
+ * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
+ * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY
  * OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
- * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS 
+ * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
  **************************************************************************/
@@ -34,7 +33,7 @@
  * \addtogroup SSDPlib
  *
  * @{
- * 
+ *
  * \file
  */
 
@@ -57,10 +56,10 @@
 
 #include <stdio.h>
 
-#ifdef WIN32
+#ifdef _WIN32
 #include <string.h>
 #define snprintf _snprintf
-#endif /* WIN32 */
+#endif /* _WIN32 */
 
 /*!
  * \brief Sends a callback to the control point application with a SEARCH
@@ -68,24 +67,28 @@
  */
 static void send_search_result(
 	/* [in] Search reply from the device. */
-	IN void *data)
+	void *data)
 {
-	ResultData *temp = (ResultData *) data;
+	SSDPResultData *temp = (SSDPResultData *)data;
 
-	temp->ctrlpt_callback(UPNP_DISCOVERY_SEARCH_RESULT, &temp->param,
-			      temp->cookie);
-	free(temp);
+	SSDPResultData_Callback(temp);
+	SSDPResultData_delete(temp);
 }
 
-void ssdp_handle_ctrlpt_msg(http_message_t *hmsg, struct sockaddr_storage *dest_addr,
-			    int timeout, void *cookie)
+void ssdp_handle_ctrlpt_msg(
+	http_message_t *hmsg,
+	struct sockaddr_storage *dest_addr,
+	int timeout)
 {
 	int handle;
+	int handle_start;
 	struct Handle_Info *ctrlpt_info = NULL;
 	memptr hdr_value;
 	/* byebye or alive */
 	int is_byebye;
-	struct Upnp_Discovery param;
+	UpnpDiscovery *param = UpnpDiscovery_new();
+	int expires;
+	int ret;
 	SsdpEvent event;
 	int nt_found;
 	int usn_found;
@@ -97,75 +100,82 @@ void ssdp_handle_ctrlpt_msg(http_message_t *hmsg, struct sockaddr_storage *dest_
 	ListNode *node = NULL;
 	SsdpSearchArg *searchArg = NULL;
 	int matched = 0;
-	ResultData *threadData = NULL;
+	SSDPResultData *threadData = NULL;
 	ThreadPoolJob job;
 
-	memset(&job, 0, sizeof(job));
-
 	/* we are assuming that there can be only one client supported at a time */
 	HandleReadLock();
 
-	if (GetClientHandleInfo(&handle, &ctrlpt_info) != HND_CLIENT) {
+	if (GetClientHandleInfo(&handle_start, &ctrlpt_info) != HND_CLIENT) {
 		HandleUnlock();
-		return;
+		goto end_ssdp_handle_ctrlpt_msg;
 	}
-	/* copy */
-	ctrlpt_callback = ctrlpt_info->Callback;
-	ctrlpt_cookie = ctrlpt_info->Cookie;
 	HandleUnlock();
 	/* search timeout */
 	if (timeout) {
-		ctrlpt_callback(UPNP_DISCOVERY_SEARCH_TIMEOUT, NULL, cookie);
-		return;
+		for (handle = handle_start; handle < NUM_HANDLE; handle++) {
+			HandleLock();
+
+			/* get client info */
+			if (GetHandleInfo(handle, &ctrlpt_info) != HND_CLIENT) {
+				HandleUnlock();
+				continue;
+			}
+			/* copy */
+			ctrlpt_callback = ctrlpt_info->Callback;
+			ctrlpt_cookie = ctrlpt_info->Cookie;
+			HandleUnlock();
+
+			ctrlpt_callback(UPNP_DISCOVERY_SEARCH_TIMEOUT, NULL, ctrlpt_cookie);
+		}
+		goto end_ssdp_handle_ctrlpt_msg;
 	}
-	param.ErrCode = UPNP_E_SUCCESS;
+
+	UpnpDiscovery_set_ErrCode(param, UPNP_E_SUCCESS);
 	/* MAX-AGE, assume error */
-	param.Expires = -1;
+	expires = -1;
+	UpnpDiscovery_set_Expires(param, expires);
 	if (httpmsg_find_hdr(hmsg, HDR_CACHE_CONTROL, &hdr_value) != NULL) {
-		if (matchstr(hdr_value.buf, hdr_value.length,
-			     "%imax-age = %d%0", &param.Expires) != PARSE_OK)
-			return;
+		ret = matchstr(hdr_value.buf, hdr_value.length,
+			"%imax-age = %d%0", &expires);
+		UpnpDiscovery_set_Expires(param, expires);
+		if (ret != PARSE_OK)
+			goto end_ssdp_handle_ctrlpt_msg;
 	}
 	/* DATE */
-	param.Date[0] = '\0';
 	if (httpmsg_find_hdr(hmsg, HDR_DATE, &hdr_value) != NULL) {
-		linecopylen(param.Date, hdr_value.buf, hdr_value.length);
+		UpnpDiscovery_strcpy_Date(param, hdr_value.buf);
 	}
 	/* dest addr */
-	memcpy(&param.DestAddr, dest_addr, sizeof(struct sockaddr_storage));
+	UpnpDiscovery_set_DestAddr(param, dest_addr);
 	/* EXT */
-	param.Ext[0] = '\0';
 	if (httpmsg_find_hdr(hmsg, HDR_EXT, &hdr_value) != NULL) {
-		linecopylen(param.Ext, hdr_value.buf, hdr_value.length);
+		UpnpDiscovery_strncpy_Ext(param, hdr_value.buf,
+					  hdr_value.length);
 	}
 	/* LOCATION */
-	param.Location[0] = '\0';
 	if (httpmsg_find_hdr(hmsg, HDR_LOCATION, &hdr_value) != NULL) {
-		linecopylen(param.Location, hdr_value.buf, hdr_value.length);
+		UpnpDiscovery_strncpy_Location(param, hdr_value.buf,
+					       hdr_value.length);
 	}
 	/* SERVER / USER-AGENT */
-	param.Os[0] = '\0';
 	if (httpmsg_find_hdr(hmsg, HDR_SERVER, &hdr_value) != NULL ||
 	    httpmsg_find_hdr(hmsg, HDR_USER_AGENT, &hdr_value) != NULL) {
-		linecopylen(param.Os, hdr_value.buf, hdr_value.length);
+		UpnpDiscovery_strncpy_Os(param, hdr_value.buf,
+					 hdr_value.length);
 	}
 	/* clear everything */
-	memset(param.DeviceId, 0, sizeof(param.DeviceId));
-	memset(param.DeviceType, 0, sizeof(param.DeviceType));
-	memset(param.ServiceType, 0, sizeof(param.ServiceType));
-	/* not used; version is in ServiceType */
-	param.ServiceVer[0] = '\0';
 	event.UDN[0] = '\0';
 	event.DeviceType[0] = '\0';
 	event.ServiceType[0] = '\0';
-	nt_found = FALSE;
+	nt_found = 0;
 	if (httpmsg_find_hdr(hmsg, HDR_NT, &hdr_value) != NULL) {
 		save_char = hdr_value.buf[hdr_value.length];
 		hdr_value.buf[hdr_value.length] = '\0';
 		nt_found = (ssdp_request_type(hdr_value.buf, &event) == 0);
 		hdr_value.buf[hdr_value.length] = save_char;
 	}
-	usn_found = FALSE;
+	usn_found = 0;
 	if (httpmsg_find_hdr(hmsg, HDR_USN, &hdr_value) != NULL) {
 		save_char = hdr_value.buf[hdr_value.length];
 		hdr_value.buf[hdr_value.length] = '\0';
@@ -173,29 +183,30 @@ void ssdp_handle_ctrlpt_msg(http_message_t *hmsg, struct sockaddr_storage *dest_
 		hdr_value.buf[hdr_value.length] = save_char;
 	}
 	if (nt_found || usn_found) {
-		strncpy(param.DeviceId, event.UDN, sizeof(param.DeviceId) - 1);
-		strncpy(param.DeviceType, event.DeviceType,
-			sizeof(param.DeviceType) - 1);
-		strncpy(param.ServiceType, event.ServiceType,
-			sizeof(param.ServiceType) - 1);
+		UpnpDiscovery_strcpy_DeviceID(param, event.UDN);
+		UpnpDiscovery_strcpy_DeviceType(param, event.DeviceType);
+		UpnpDiscovery_strcpy_ServiceType(param, event.ServiceType);
 	}
 	/* ADVERT. OR BYEBYE */
 	if (hmsg->is_request) {
 		/* use NTS hdr to determine advert., or byebye */
 		if (httpmsg_find_hdr(hmsg, HDR_NTS, &hdr_value) == NULL) {
-			return;	/* error; NTS header not found */
+			/* error; NTS header not found */
+			goto end_ssdp_handle_ctrlpt_msg;
 		}
 		if (memptr_cmp(&hdr_value, "ssdp:alive") == 0) {
-			is_byebye = FALSE;
+			is_byebye = 0;
 		} else if (memptr_cmp(&hdr_value, "ssdp:byebye") == 0) {
-			is_byebye = TRUE;
+			is_byebye = 1;
 		} else {
-			return;	/* bad value */
+			/* bad value */
+			goto end_ssdp_handle_ctrlpt_msg;
 		}
 		if (is_byebye) {
 			/* check device byebye */
 			if (!nt_found || !usn_found) {
-				return;	/* bad byebye */
+				/* bad byebye */
+				goto end_ssdp_handle_ctrlpt_msg;
 			}
 			event_type = UPNP_DISCOVERY_ADVERTISEMENT_BYEBYE;
 		} else {
@@ -204,17 +215,33 @@ void ssdp_handle_ctrlpt_msg(http_message_t *hmsg, struct sockaddr_storage *dest_
 			 * only. Expires should be greater than 1800 (30 mins) */
 			if (!nt_found ||
 			    !usn_found ||
-			    strlen(param.Location) == 0 || param.Expires <= 0) {
-				return;	/* bad advertisement */
+			    UpnpString_get_Length(UpnpDiscovery_get_Location(param)) == 0 ||
+			    UpnpDiscovery_get_Expires(param) <= 0) {
+				/* bad advertisement */
+				goto end_ssdp_handle_ctrlpt_msg;
 			}
 			event_type = UPNP_DISCOVERY_ADVERTISEMENT_ALIVE;
 		}
 		/* call callback */
-		ctrlpt_callback(event_type, &param, ctrlpt_cookie);
+		for (handle = handle_start; handle < NUM_HANDLE; handle++) {
+			HandleLock();
+
+			/* get client info */
+			if (GetHandleInfo(handle, &ctrlpt_info) != HND_CLIENT) {
+				HandleUnlock();
+				continue;
+			}
+			/* copy */
+			ctrlpt_callback = ctrlpt_info->Callback;
+			ctrlpt_cookie = ctrlpt_info->Cookie;
+			HandleUnlock();
+
+			ctrlpt_callback(event_type, param, ctrlpt_cookie);
+		}
 	} else {
 		/* reply (to a SEARCH) */
 		/* only checking to see if there is a valid ST header */
-		st_found = FALSE;
+		st_found = 0;
 		if (httpmsg_find_hdr(hmsg, HDR_ST, &hdr_value) != NULL) {
 			save_char = hdr_value.buf[hdr_value.length];
 			hdr_value.buf[hdr_value.length] = '\0';
@@ -223,85 +250,102 @@ void ssdp_handle_ctrlpt_msg(http_message_t *hmsg, struct sockaddr_storage *dest_
 			hdr_value.buf[hdr_value.length] = save_char;
 		}
 		if (hmsg->status_code != HTTP_OK ||
-		    param.Expires <= 0 ||
-		    strlen(param.Location) == 0 || !usn_found || !st_found) {
-			return;	/* bad reply */
+		    UpnpDiscovery_get_Expires(param) <= 0 ||
+		    UpnpString_get_Length(UpnpDiscovery_get_Location(param)) == 0 ||
+		    !usn_found || !st_found) {
+			/* bad reply */
+			goto end_ssdp_handle_ctrlpt_msg;
 		}
 		/* check each current search */
-		HandleLock();
-		if (GetClientHandleInfo(&handle, &ctrlpt_info) != HND_CLIENT) {
-			HandleUnlock();
-			return;
-		}
-		node = ListHead(&ctrlpt_info->SsdpSearchList);
-		/* temporary add null termination */
-		/*save_char = hdr_value.buf[ hdr_value.length ]; */
-		/*hdr_value.buf[ hdr_value.length ] = '\0'; */
-		while (node != NULL) {
-			searchArg = node->item;
-			/* check for match of ST header and search target */
-			switch (searchArg->requestType) {
-			case SSDP_ALL:
-				matched = 1;
-				break;
-			case SSDP_ROOTDEVICE:
-				matched =
-				    (event.RequestType == SSDP_ROOTDEVICE);
-				break;
-			case SSDP_DEVICEUDN:
-				matched = !strncmp(searchArg->searchTarget,
-						   hdr_value.buf,
-						   hdr_value.length);
-				break;
-			case SSDP_DEVICETYPE:{
-					size_t m = min(hdr_value.length,
-						       strlen
-						       (searchArg->searchTarget));
-					matched =
-					    !strncmp(searchArg->searchTarget,
-						     hdr_value.buf, m);
-					break;
-				}
-			case SSDP_SERVICE:{
-					size_t m = min(hdr_value.length,
-						       strlen
-						       (searchArg->searchTarget));
-					matched =
-					    !strncmp(searchArg->searchTarget,
-						     hdr_value.buf, m);
-					break;
-				}
-			default:
-				matched = 0;
-				break;
+		for (handle = handle_start; handle < NUM_HANDLE; handle++) {
+			HandleLock();
+
+			/* get client info */
+			if (GetHandleInfo(handle, &ctrlpt_info) != HND_CLIENT) {
+				HandleUnlock();
+				continue;
 			}
-			if (matched) {
-				/* schedule call back */
-				threadData =
-				    (ResultData *) malloc(sizeof(ResultData));
-				if (threadData != NULL) {
-					threadData->param = param;
-					threadData->cookie = searchArg->cookie;
-					threadData->ctrlpt_callback =
-					    ctrlpt_callback;
-					TPJobInit(&job, (start_routine)
-						  send_search_result,
-						  threadData);
-					TPJobSetPriority(&job, MED_PRIORITY);
-					TPJobSetFreeFunction(&job,
-							     (free_routine)
-							     free);
-					if (ThreadPoolAdd(&gRecvThreadPool, &job, NULL) != 0) {
-						free(threadData);
+			/* copy */
+			ctrlpt_callback = ctrlpt_info->Callback;
+			ctrlpt_cookie = ctrlpt_info->Cookie;
+
+			node = ListHead(&ctrlpt_info->SsdpSearchList);
+			/* temporary add null termination */
+			/*save_char = hdr_value.buf[ hdr_value.length ]; */
+			/*hdr_value.buf[ hdr_value.length ] = '\0'; */
+			while (node != NULL) {
+				searchArg = node->item;
+				/* check for match of ST header and search target */
+				switch (searchArg->requestType) {
+				case SSDP_ALL:
+					matched = 1;
+					break;
+				case SSDP_ROOTDEVICE:
+					matched =
+					    (event.RequestType == SSDP_ROOTDEVICE);
+					break;
+				case SSDP_DEVICEUDN:
+					matched = !strncmp(searchArg->searchTarget,
+							   hdr_value.buf,
+							   hdr_value.length);
+					break;
+				case SSDP_DEVICETYPE:{
+						size_t m = min(hdr_value.length,
+							       strlen
+							       (searchArg->searchTarget));
+						matched =
+						    !strncmp(searchArg->searchTarget,
+							     hdr_value.buf, m);
+						break;
+					}
+				case SSDP_SERVICE:{
+						size_t m = min(hdr_value.length,
+							       strlen
+							       (searchArg->searchTarget));
+						matched =
+						    !strncmp(searchArg->searchTarget,
+							     hdr_value.buf, m);
+						break;
+					}
+				default:
+					matched = 0;
+					break;
+				}
+				if (matched) {
+					/* schedule call back */
+					threadData = SSDPResultData_new();
+					if (threadData != NULL) {
+						SSDPResultData_set_Param(threadData,
+									 param);
+						SSDPResultData_set_Cookie(threadData,
+									  searchArg->
+									  cookie);
+						SSDPResultData_set_CtrlptCallback
+						    (threadData, ctrlpt_callback);
+						memset(&job, 0, sizeof(job));
+
+						TPJobInit(&job, (start_routine)
+							  send_search_result,
+							  threadData);
+						TPJobSetPriority(&job, MED_PRIORITY);
+						TPJobSetFreeFunction(&job,
+								     (free_routine)
+								     free);
+						if (ThreadPoolAdd(&gRecvThreadPool, &job, NULL) != 0) {
+							SSDPResultData_delete(threadData);
+						}
 					}
 				}
+				node = ListNext(&ctrlpt_info->SsdpSearchList, node);
 			}
-			node = ListNext(&ctrlpt_info->SsdpSearchList, node);
-		}
 
-		HandleUnlock();
-		/*ctrlpt_callback( UPNP_DISCOVERY_SEARCH_RESULT, &param, cookie ); */
+			HandleUnlock();
+			/*ctrlpt_callback( UPNP_DISCOVERY_SEARCH_RESULT, param, ctrlpt_cookie ); */
+		}
 	}
+
+end_ssdp_handle_ctrlpt_msg:
+	UpnpDiscovery_delete(param);
 }
 
 /*!
@@ -310,15 +354,15 @@ void ssdp_handle_ctrlpt_msg(http_message_t *hmsg, struct sockaddr_storage *dest_
  */
 static int CreateClientRequestPacket(
 	/*! [in,out] Output string in HTTP format. */
-	INOUT char *RqstBuf,
+	char *RqstBuf,
 	/*! [in] RqstBuf size. */
-	IN size_t RqstBufSize,
+	size_t RqstBufSize,
 	/*! [in] Search Target. */
-	IN int Mx,
+	int Mx,
 	/*! [in] Number of seconds to wait to collect all the responses. */
-	IN char *SearchTarget,
+	char *SearchTarget,
 	/*! [in] search address family. */
-	IN int AddressFamily)
+	int AddressFamily)
 {
 	int rc;
 	char TempBuf[COMMAND_LEN];
@@ -456,9 +500,8 @@ static void searchExpired(
 	/* [in] . */
 	void *arg)
 {
-
-	int *id = (int *)arg;
-	int handle = -1;
+	int id = ((SsdpSearchExpArg *)arg)->timeoutEventId;
+	int handle = ((SsdpSearchExpArg *)arg)->handle;
 	struct Handle_Info *ctrlpt_info = NULL;
 
 	/* remove search Target from list and call client back */
@@ -471,8 +514,8 @@ static void searchExpired(
 	HandleLock();
 
 	/* remove search target from search list */
-	if (GetClientHandleInfo(&handle, &ctrlpt_info) != HND_CLIENT) {
-		free(id);
+	if (GetHandleInfo(handle, &ctrlpt_info) != HND_CLIENT) {
+		free(arg);
 		HandleUnlock();
 		return;
 	}
@@ -480,7 +523,7 @@ static void searchExpired(
 	node = ListHead(&ctrlpt_info->SsdpSearchList);
 	while (node != NULL) {
 		item = (SsdpSearchArg *) node->item;
-		if (item->timeoutEventId == (*id)) {
+		if (item->timeoutEventId == id) {
 			free(item->searchTarget);
 			cookie = item->cookie;
 			found = 1;
@@ -496,10 +539,10 @@ static void searchExpired(
 	if (found)
 		ctrlpt_callback(UPNP_DISCOVERY_SEARCH_TIMEOUT, NULL, cookie);
 
-	free(id);
+	free(arg);
 }
 
-int SearchByTarget(int Mx, char *St, void *Cookie)
+int SearchByTarget(int Hnd, int Mx, char *St, void *Cookie)
 {
 	char errorBuffer[ERROR_BUFFER_LEN];
 	int *id = NULL;
@@ -519,17 +562,21 @@ int SearchByTarget(int Mx, char *St, void *Cookie)
 #endif
 	fd_set wrSet;
 	SsdpSearchArg *newArg = NULL;
+	SsdpSearchExpArg *expArg = NULL;
 	int timeTillRead = 0;
-	int handle;
 	struct Handle_Info *ctrlpt_info = NULL;
 	enum SsdpSearchType requestType;
-	unsigned long addrv4 = inet_addr(gIF_IPV4);
+	struct in_addr addrv4;
 	SOCKET max_fd = 0;
 	int retVal;
 
 	/*ThreadData *ThData; */
 	ThreadPoolJob job;
 
+	if (!inet_aton(gIF_IPV4, &addrv4)) {
+		return UPNP_E_INVALID_PARAM;
+	}
+
 	memset(&job, 0, sizeof(job));
 
 	requestType = ssdp_request_type1(St);
@@ -569,7 +616,7 @@ int SearchByTarget(int Mx, char *St, void *Cookie)
 
 	/* add search criteria to list */
 	HandleLock();
-	if (GetClientHandleInfo(&handle, &ctrlpt_info) != HND_CLIENT) {
+	if (GetHandleInfo(Hnd, &ctrlpt_info) != HND_CLIENT) {
 		HandleUnlock();
 		return UPNP_E_INTERNAL_ERROR;
 	}
@@ -577,8 +624,11 @@ int SearchByTarget(int Mx, char *St, void *Cookie)
 	newArg->searchTarget = strdup(St);
 	newArg->cookie = Cookie;
 	newArg->requestType = requestType;
-	id = (int *)malloc(sizeof(int));
-	TPJobInit(&job, (start_routine) searchExpired, id);
+
+	expArg = (SsdpSearchExpArg *)malloc(sizeof(SsdpSearchExpArg));
+	expArg->handle = Hnd;
+	id = (int *)&(expArg->timeoutEventId);
+	TPJobInit(&job, (start_routine) searchExpired, expArg);
 	TPJobSetPriority(&job, MED_PRIORITY);
 	TPJobSetFreeFunction(&job, (free_routine) free);
 	/* Schedule a timeout event to remove search Arg */
